// camera class
// -Kyoung Shin Park (2008 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef __CAMERA_H__
#define __CAMERA_H__

#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "vector3.h"
#include "matrix4x4.h"

namespace oglclass {

enum cameraType { DRIVE, FLY };

class camera
{
public:
	camera(double fovy = 45.0, double aspect = 1.0,
           double znear = 1.0, double zfar = 100.0, cameraType type = DRIVE);
	~camera();

	void apply(void);
	void applyProjection(void);
	void setAspect(double);

	void strafe(float units); // left/right
	void fly(float units);    // up/down
	void walk(float units);   // forward/backward
	
	void pitch(float angle); // rotate on right vector
	void yaw(float angle);   // rotate on up vector
	void roll(float angle);  // rotate on look vector

	void lookAt(vector3& eye, vector3& at, vector3& up);

	void setCameraType(cameraType type);
	void setPosition(float x, float y, float z);
	void setPosition(const vector3& pos); 
	void setAt(float x, float y, float z);
	void setAt(const vector3& pos); 

	void setRight(float x=1.0, float y=0.0, float z=0.0);
	void setRight(const vector3& pos);
	void setUp(float x=0.0, float y=1.0, float z=0.0);
	void setUp(const vector3& pos);
	void setLook(float x=0.0, float y=0.0, float z=1.0);
	void setLook(const vector3& pos);

	vector3 getPosition(); 
	vector3 getRight();
	vector3 getUp();
	vector3 getLook();

private:
    double fovy_, aspect_;
    double near_, far_;

	cameraType  cameraType_;

	vector3 pos_;
	vector3 right_;
	vector3 up_;
	vector3 look_;
};

} // namespace

#endif // __CAMERA_H__