// font class
// -Kyoung Shin Park (2009 Spring)
//////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <GL/glut.h>
#include <stdio.h>
#include <stdarg.h>
#include "font.h"

#define WIN32_MEAN_AND_LEAN
#define WIN32_EXTRA_LEAN

namespace oglclass
{

font::font()
{
	screenX_ = screenY_ = 0;
	x_ = y_ = z_ = 0.0;
	r_ = g_ = b_ = a_ = 1.0; 
	strcpy(name_, "GLUT_BITMAP_TIMES_ROMAN_24");
}

font::font(char *name)
{
	screenX_ = screenY_ = 0;
	x_ = y_ = z_ = 0.0;
	r_ = g_ = b_ = a_ = 1.0; 
	strcpy(name_, name);
}

font::~font()
{
	glDeleteLists(callList_, 96);
}

void font::build(char *name, int size)
{
	HFONT hFont;	// font ID
	HDC hDC;		// device context

	hDC = wglGetCurrentDC();
	callList_ = glGenLists(96);

	if (stricmp(name, "symbol") == 0)
	{
		hFont = CreateFont(-size, 0,0,0,FW_BOLD, FALSE, FALSE, FALSE, SYMBOL_CHARSET,
						OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
						FF_DONTCARE | DEFAULT_PITCH, name);
	}
	else
	{
		hFont = CreateFont(-size, 0,0,0,FW_BOLD, FALSE, FALSE, FALSE, ANSI_CHARSET,
					OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, 
					FF_DONTCARE | DEFAULT_PITCH, name);
	}

	SelectObject(hDC, hFont);
	wglUseFontBitmaps(hDC, 32, 96, callList_);
}

void font::clearFont()
{
	glDeleteLists(callList_, 96);
}

void font::print(const char *str, ...)
{
	char text[256];
	va_list args;

	if (str == NULL)
		return;

	va_start(args, str);
		vsprintf(text, str, args);
	va_end(args);

	glPushMatrix();
	glColor4f(r_, g_, b_, a_);
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -1.0f);	// translate one unit into the screen
	if (x_ == 0.0 && y_ == 0.0 && z_ == 0.0)
		glRasterPos2i(screenX_, screenY_);
	else
		glRasterPos3f(x_, y_, z_);

	glPushAttrib(GL_LIST_BIT);
		glListBase(callList_ - 32);
		glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
	glPopAttrib();
	glPopMatrix();
}


void drawBitmapString(char* str, GLfloat x, GLfloat y) // draw text
{
	glRasterPos3f(x, y, 0);
	int l = strlen(str);
	int i = 0;
	while(i < l) {
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, str[i++]);
	}
}

// PLEASE READ THIS ARTICLE http://www.lighthouse3d.com/opengl/glut/index.php?bmpfontortho
// draw text in ortho projection
void font::draw(int x, int y, char * str, ...)
{
	if (str == NULL)
		return;

	// set orthographic projection
	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);							// Select The Projection Matrix
	glPushMatrix();											// Store The Projection Matrix
	glLoadIdentity();										// Reset The Projection Matrix
	gluOrtho2D(0, viewport[2], 0, viewport[3]);				// Set Up An Ortho Screen
	glScalef(1, -1, 1);
	glTranslatef(0, -viewport[3], 0);
	glMatrixMode(GL_MODELVIEW);								// Select The Modelview Matrix

		glPushMatrix();
		glLoadIdentity();

		char text[255];
		va_list args;											// Pointer To List Of Arguments
		va_start(args, str);										// Parses The String For Variables
		    vsprintf(text, str, args);								// And Converts Symbols To Actual Numbers
		va_end(args);													// Results Are Stored In Text

        glPushAttrib(GL_ALL_ATTRIB_BITS);
		glColor4f(r_, g_, b_, a_);
		drawBitmapString(text, x, y);
		glPopAttrib();

		glPopMatrix();

	// reset perspective projection
	glMatrixMode(GL_PROJECTION);							// Select The Projection Matrix
	glPopMatrix();											// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);								// Select The Modelview Matrix
}

} // namespace