#include "light.h"

namespace oglclass
{

int light::nextLightNum_ = 0;


light::light(void)
{
    if (nextLightNum_ < GL_MAX_LIGHTS)
    {
        lightNumber_ = GL_LIGHT0 + nextLightNum_;
        nextLightNum_++;
    }
    else
    {
        std::cerr << "Warning: Maximum number of lights exceeded "
            "(maximum is " << GL_MAX_LIGHTS << ")\n";
        lightNumber_ = GL_LIGHT0 + GL_MAX_LIGHTS - 1;
    }
    ambient_.set(0,0,0,1);
    diffuse_.set(1,1,1,1);
    specular_.set(1,1,1,1);
    position_.set(0,0,1,0);
    attenuationConstant_ = 1;
    attenuationLinear_ = 0;
    attenuationQuadratic_ = 0;
    spotDirection_.set(0,0,-1);
    spotCutoff_ = 180;
    spotExponent_ = 0;
}

void light::enable(void)
{
    glEnable(lightNumber_);
	glLightfv(lightNumber_, GL_POSITION, position_.vec);
	glLightfv(lightNumber_, GL_AMBIENT, ambient_.vec);
	glLightfv(lightNumber_, GL_DIFFUSE, diffuse_.vec);
	glLightfv(lightNumber_, GL_SPECULAR, specular_.vec);
    glLightf(lightNumber_, GL_CONSTANT_ATTENUATION, attenuationConstant_);
    glLightf(lightNumber_, GL_LINEAR_ATTENUATION, attenuationLinear_);
    glLightf(lightNumber_, GL_QUADRATIC_ATTENUATION, attenuationQuadratic_);
    glLightf(lightNumber_, GL_SPOT_CUTOFF, spotCutoff_);
    if (spotCutoff_ < 180)
    {
        glLightfv(lightNumber_, GL_SPOT_DIRECTION, spotDirection_.vec);
        glLightf(lightNumber_, GL_SPOT_EXPONENT, spotExponent_);
    }
}

void light::disable(void)
{
    glDisable(lightNumber_);
}

void light::setAmbient(const vector4& color)
{
    ambient_ = color;
}

void light::setAmbient(float r, float g, float b, float a)
{
    ambient_[0] = r;
    ambient_[1] = g;
    ambient_[2] = b;
    ambient_[3] = a;
}

void light::setDiffuse(const vector4& color)
{
    diffuse_ = color;
}

void light::setDiffuse(float r, float g, float b, float a)
{
    diffuse_[0] = r;
    diffuse_[1] = g;
    diffuse_[2] = b;
    diffuse_[3] = a;
}

void light::setSpecular(const vector4& color)
{
    specular_ = color;
}

void light::setSpecular(float r, float g, float b, float a)
{
    specular_[0] = r;
    specular_[1] = g;
    specular_[2] = b;
    specular_[3] = a;
}

void light::setPosition(vector4& position)
{
    position_ = position;
}

void light::setPosition(float x, float y, float z, float w)
{
    position_[0] = x;
    position_[1] = y;
    position_[2] = z;
    position_[3] = w;
}

void light::setInfinitePosition(vector3& position)
{
    setPosition(position[0], position[1], position[2], 0);
}

void light::setInfinitePosition(float x, float y, float z)
{
    setPosition(x, y, z, 0);
}

void light::setLocalPosition(vector3& position)
{
    setPosition(position[0], position[1], position[2], 1);
}

void light::setLocalPosition(float x, float y, float z)
{
    setPosition(x, y, z, 1);
}

void light::setAttenuation(float constant, float linear, float quadratic)
{
    attenuationConstant_ = constant;
    attenuationLinear_ = linear;
    attenuationQuadratic_ = quadratic;
}

void light::setSpotDirection(const vector3& direction)
{
    spotDirection_ = direction;
}

void light::setSpotCutoff(GLfloat cutoff)
{
    spotCutoff_ = cutoff;
}

void light::setSpotExponent(GLfloat exponent)
{
    spotExponent_ = exponent;
}

} // namespace
