#ifndef __LIGHT_H__
#define __LIGHT_H__

#include <windows.h>
#include <GL/gl.h>
#include "vector3.h"
#include "vector4.h"

namespace oglclass
{

class light
{
    public:
        light(void);
        void enable(void);
        void disable(void);
        
        void setAmbient(const vector4& color);
        void setDiffuse(const vector4& color);
        void setSpecular(const vector4& color);

        void setAmbient(float r, float g, float b, float a);
        void setDiffuse(float r, float g, float b, float a);
        void setSpecular(float r, float g, float b, float a);

		void setPosition(vector4& position);
        void setPosition(float x, float y, float z, float w);
		void setInfinitePosition(vector3& position);
        void setInfinitePosition(float x, float y, float z);
		void setLocalPosition(vector3& position);
        void setLocalPosition(float x, float y, float z);

		void setAttenuation(float constant, float linear, float quadratic);
        void setSpotDirection(const vector3& direction);
        void setSpotCutoff(float cutoff);
        void setSpotExponent(float exponent);
    
        vector4& ambient(void) { return ambient_; }
        vector4& diffuse(void) { return diffuse_; }
        vector4& specular(void) { return specular_; }
        vector4& position(void) { return position_; }
        float attenuationConstant(void) { return attenuationConstant_; }
        float attenuationLinear(void) { return attenuationLinear_; }
        float attenuationQuadratic(void) { return attenuationQuadratic_; }
        float spotCutoff(void) { return spotCutoff_; }
        vector3& spotDirection(void) { return spotDirection_; }
        float spotExponent(void) { return spotExponent_; }

    private:
        GLenum lightNumber_;
        vector4 ambient_, diffuse_, specular_;
        vector4 position_;
        float attenuationConstant_, attenuationLinear_, attenuationQuadratic_;
        vector3 spotDirection_;
        float spotCutoff_, spotExponent_;
        
        static int nextLightNum_;
};

} // namespace

#endif
