#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "oglclass.h"

using namespace oglclass;

float  g_ElapsedTime = 0.;
double g_CurrentTime = 0.;
double g_PreviousTime = 0.;
double g_DeltaTime = 0.;
int g_NumSoundPlayed = 0;

// sound
sound sounds;
ALfloat listnerPos[] = { 0., 0., 0.};
ALfloat listnerVel[] = { 0., 0., 0.};
ALfloat listnerOri[] = { 0., 0., -1., 0., 1., 0.}; // first three elements are "at", second three are "up"
ALfloat sourcePos1[] = { 10., 5., 0.};
ALfloat sourceVel1[] = { 0., 0., 0.};
ALfloat sourcePos2[] = { -10., 5., -5.};
ALfloat sourceVel2[] = { 0., 0., 0.};

// camera
camera camera1(45, 1, 0.1f, 1000.0f, FLY);

// light
light light0;
light light1;

// font
font text("GLUT_BITMAP_TIMES_ROMAN_24");

// obj loader
int g_obj, g_obj2;
transform objTrans;

void init(void)
{
	glClearColor( 0.325f, 0.3f, 0.4f, 1.0f );
	glEnable(GL_DEPTH_TEST);

	light0.setInfinitePosition(1, 1, 0);
	light0.setDiffuse(1, 0.5, 0, 1);
	light0.enable();

	light1.setInfinitePosition(-1, 1, 0);
	light1.enable();

	camera1.applyProjection();
	camera1.setPosition(0.0, 2.0, 25.0);

	// openal sound
	sounds.init();
	sounds.setListner(listnerPos, listnerVel, listnerOri);
	sounds.setSoundSources("sound/darkside_of_the_moon.wav", sourcePos1, sourceVel1, AL_TRUE);
	//sounds.play(0);
	sounds.setSoundSources("sound/shot.wav", sourcePos2, sourceVel2, AL_FALSE);

	// obj model file loading
	g_obj = obj_add_file("model/miro.obj");
	g_obj2 = obj_add_file("model/car_m.obj");
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case 27:
			exit(0);
			break;

		case ' ':
			sounds.play(1);
			g_NumSoundPlayed++;
			break;

		default:
			break;
	}
}

void specialkey(int key, int x, int y)
{
	if (key == GLUT_KEY_F1)			// x-movement
        camera1.strafe(0.5);
	else if (key == GLUT_KEY_F2)
        camera1.strafe(-0.5);
	else if (key == GLUT_KEY_F3)	// y-movement
        camera1.fly(0.5);
	else if (key == GLUT_KEY_F4)
        camera1.fly(-0.5);
	else if (key == GLUT_KEY_F5)	// z-movement
        camera1.walk(0.5);
    else if (key == GLUT_KEY_F6)
        camera1.walk(-0.5);

	else if (key == GLUT_KEY_F7)	// yaw (by y-axis)
        camera1.yaw(2.5);
	else if (key == GLUT_KEY_F8)
        camera1.yaw(-2.5);
	else if (key == GLUT_KEY_F9)	// pitch (by x-axis)
        camera1.pitch(2.5);
    else if (key == GLUT_KEY_F10)
        camera1.pitch(-2.5);
	else if (key == GLUT_KEY_HOME)	// roll (by z-axis)
        camera1.roll(2.5);
    else if (key == GLUT_KEY_END)
        camera1.roll(-2.5);

	else if (key == GLUT_KEY_LEFT)	// same as town
        camera1.yaw(2.5);
	else if (key == GLUT_KEY_RIGHT)
        camera1.yaw(-2.5);
	else if (key == GLUT_KEY_UP)	
        camera1.walk(0.5);
    else if (key == GLUT_KEY_DOWN)
        camera1.walk(-0.5);

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	camera1.setAspect((float)w / (float)h);
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	camera1.apply();

	// draw obj model
	obj_draw_file(g_obj);

	objTrans.pushMatrix();
	objTrans.setScaling(0.1, 0.1, 0.1);
	obj_draw_file(g_obj2);
	objTrans.popMatrix();

	// text
	text.setColor(1.0, 1.0, 0.0);
	text.draw(20, 50, "Number of Sound Played: %i\n", g_NumSoundPlayed);

	glutSwapBuffers();
}

void update()
{
	g_CurrentTime = timeGetTime();
	g_DeltaTime = g_CurrentTime - g_PreviousTime;
	g_ElapsedTime = (float)(g_DeltaTime * 0.001);
	g_PreviousTime = g_CurrentTime;

	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);

	init();

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialkey);
	glutIdleFunc(update);
	glEnable(GL_DEPTH_TEST);
	glutMainLoop();

	return 0;
}
