#include "material.h"

namespace oglclass
{

material::material(void)
{
    ambient_.set(0.2f, 0.2f, 0.2f, 1.0f);
    diffuse_.set(0.8f, 0.8f, 0.8f, 1.0f);
    specular_.set(0, 0, 0, 1);
    emission_.set(0, 0, 0, 1);
    shininess_ = 0;
}

material::material(const vector4& color)
{
    ambient_ = color;
    diffuse_ = color;
    specular_.set(0, 0, 0, 1);
    emission_.set(0, 0, 0, 1);
    shininess_ = 0;
}

material::material(const vector4& ambientAndDiffuse, const vector4& specular, float shininess)
{
    ambient_ = ambientAndDiffuse;
    diffuse_ = ambientAndDiffuse;
    specular_ = specular;
    emission_.set(0, 0, 0, 1);
    shininess_ = shininess;
}

void material::enable(void)
{
    glEnable(GL_LIGHTING);
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient_.vec);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse_.vec);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular_.vec);
    glMaterialfv(GL_FRONT, GL_EMISSION, emission_.vec);
    glMaterialf(GL_FRONT, GL_SHININESS, shininess_);
}

void material::disable(void)
{
    glDisable(GL_LIGHTING);
}

void material::setAmbient(const vector4& color)
{
    ambient_ = color;
}

void material::setDiffuse(const vector4& color)
{
    diffuse_ = color;
}

void material::setAmbientAndDiffuse(const vector4& color)
{
    setAmbient(color);
    setDiffuse(color);
}

void material::setSpecular(const vector4& color)
{
    specular_ = color;
}

void material::setEmission(const vector4& color)
{
    emission_ = color;
}

void material::setShininess(float shininess)
{
    shininess_ = shininess;
}


} // namespace
