#include <stdio.h>
#include <malloc.h>
#include "node.h"

namespace oglclass
{

node::node(void)
{
	material_ = 0;
	texture_ = 0;
	transform_ = 0;
	textureTransform_ = 0;
	parent_ = 0;
	numChildren_ = 0;
	maxChildren_ = 16;
	children_ = (node **) malloc(maxChildren_ * sizeof(node *));
	drawFunc_ = 0;
	drawData_ = 0;
	updateFunc_ = 0;
	updateData_ = 0;
}


node::~node(void)
{
}


void node::drawAll(void)
{
	if (transform_)
		transform_->pushMatrix();
	if (material_)
		material_->enable();
	if (texture_)
		texture_->enable();
	if (textureTransform_)
		{
		glMatrixMode(GL_TEXTURE);
		 glLoadIdentity();
		 textureTransform_->pushMatrix();
		glMatrixMode(GL_MODELVIEW);
		}
	draw();
	if (textureTransform_)
		{
		glMatrixMode(GL_TEXTURE);
		 glLoadIdentity();
		glMatrixMode(GL_MODELVIEW);
		}
	if (texture_)
		texture_->disable();
	if (material_)
		material_->disable();
	for (int i=0; i < numChildren_; i++)
		children_[i]->drawAll();
	if (transform_)
		transform_->popMatrix();
}

void node::updateAll(void)
{
	update();
	for (int i=0; i < numChildren_; i++)
		children_[i]->updateAll();
}

void node::draw(void)
{
	if (drawFunc_)
		(*drawFunc_)(*this,drawData_);
}

void node::update(void)
{
	if (updateFunc_)
		(*updateFunc_)(*this,updateData_);
}


void node::setTransform(transform& t)
{
	transform_ = &t;
}


void node::setTextureTransform(transform& t)
{
	textureTransform_ = &t;
}


void node::setMaterial(material& m)
{
	material_ = &m;
}


void node::setTexture(texture& t)
{
	texture_ = &t;
}

void node::attach(node& child)
{
	if (numChildren_ == maxChildren_) {
		maxChildren_ *= 2;
		node ** newchildren = (node **)
						realloc(children_, maxChildren_ * sizeof(node *));
		if (newchildren)
			children_ = newchildren;
		else {
			std::cerr << "ERROR: node::attach() failed to realloc space for "
				 << maxChildren_ << " children\n";
			perror("realloc");
			maxChildren_ /= 2;
			return;
			}
		}
	if (child.parent_)
		child.parent_->detach(child);
	child.parent_ = this;
	children_[numChildren_] = &child;
	numChildren_++;
}

void node::detach(node& child)
{
	int i;
	for (i=0; i < numChildren_; i++) {
		if (children_[i] == &child) {
			child.parent_ = 0;
			for (; i < numChildren_-1; i++)
				children_[i] = children_[i+1];
			numChildren_--;
			break;
		}
	}
}

int node::numChildren(void)
{
	return numChildren_;
}


node& node::child(int n)
{
	if ((n < 0) || (n >= numChildren_))
		return *this;
	return *(children_[n]);
}


void node::setDrawCallback(CallbackFunc drawfunc,void *drawdata)
{
	drawFunc_ = drawfunc;
	drawData_ = drawdata;
}

void node::setUpdateCallback(CallbackFunc updatefunc,void *updatedata)
{
	updateFunc_ = updatefunc;
	updateData_ = updatedata;
}


}
