#ifndef __OBJECT_H__
#define __OBJECT_H__

#include "material.h"
#include "texture2D.h"
#include "transform.h"

namespace oglclass
{

class node 
{
public:
	typedef void (*CallbackFunc)(node&,void*);

	node(void);
	virtual ~node(void);
	void drawAll(void);
	void updateAll(void);

	virtual void draw(void);
	virtual void update(void);

	void setTransform(transform&);
	void setTextureTransform(transform&);
	void setMaterial(material&);
	void setTexture(texture&);
	transform& getTransform(void) const { return *transform_; }
	transform& getTextureTransform(void) const { return *textureTransform_; }
	material& getMaterial(void) const { return *material_; }
	texture& getTexture(void) const { return *texture_; }
    
	void attach(node&);
	void detach(node&);
	int numChildren(void);
	node& child(int n);

	void setDrawCallback(CallbackFunc drawfunc,void *data=0);
	void setDrawCallbackData(void *data);
	void setUpdateCallback(CallbackFunc updatefunc,void *data=0);
	void setUpdateCallbackData(void *data);

private:
	material * material_;
	texture * texture_;
	transform * transform_;
	transform * textureTransform_;
	node * parent_;
	node ** children_;
	int numChildren_, maxChildren_;
	CallbackFunc drawFunc_, updateFunc_;
	void *drawData_, *updateData_;
};

}

#endif
