#include "plane.h"

namespace oglclass
{

plane::plane( const plane &other )
{
    set(other.a, other.b, other.c, other.d);
}

plane::plane( const float *other )
{
    set(other[0], other[1], other[2], other[3]);
}

plane::plane( float a_, float b_, float c_, float d_)
{
    set(a_, b_, c_, d_);
}

plane::plane( vector3 p0_, vector3 p1_, vector3 p2_ )
{
    set(p0_, p1_, p2_);
}

plane::plane( vector3 n_, vector3 p_ )
{
    set(n_, p_);
}

void plane::set( float a_, float b_, float c_, float d_ )
{
    a = a_;
    b = b_;
    c = c_;
    d = d_;
    normalize();
}

void plane::set( vector3 p0_, vector3 p1_, vector3 p2_ )
{
	vector3 u, v, n;
	u = p2_ - p0_;
	v = p1_ - p0_;
    n = vector3::crossProduct(u, v);
	n.normalize();
	a = n.vec[0];
	b = n.vec[1];
	c = n.vec[2];
	d = -vector3::dotProduct(n, p0_);
}

void plane::set( vector3 n_, vector3 p_ )
{
	a = n_.vec[0];
	b = n_.vec[1];
	c = n_.vec[2];
	d = -vector3::dotProduct(n_, p_);
}

float plane::length( void )
{
    return( (float)sqrt( a * a + b * b + c * c + d * d) );
}

void plane::normalize( void )
{
    float fLength = length();

    a = a / fLength;
    b = b / fLength;
    c = c / fLength;
    d = d / fLength;
}

// a*x + b*y + c*z + d*0
// n . v (where n = [a, b, c])
float plane::dotNormal( const plane &p,  const vector3 &v )
{
    return ( p.a * v.vec[0] + p.b * v.vec[1] + p.c * v.vec[2] );
}

// a*x + b*y + c*z + d*1
// n . v + d (where n = [a, b, c])
float plane::dotCoord( const plane &p,  const vector3 &v )
{
    return ( p.a * v.vec[0] + p.b * v.vec[1] + p.c * v.vec[2] + p.d );
}

// the closest point vResult = vec - (n * vec + d) * n (where n = [a, b, c])
// dotCoord is the closest distance from point to plane
vector3& plane::closestPointOnPlane( const plane &p, const vector3 &v)
{
    vector3 vResult;

    vResult.vec[0] = v.vec[0] - dotCoord(p, v) * p.a;
    vResult.vec[1] = v.vec[1] - dotCoord(p, v) * p.b;
    vResult.vec[2] = v.vec[2] - dotCoord(p, v) * p.c;

    return vResult;
}

// operators...
plane& plane::operator = ( const plane &other )
{
    a = other.a;
    b = other.b;
    c = other.c;
    d = other.d;

    return *this;
}

bool plane::operator == ( const plane &other) const
{
	return (a == other.a && b == other.b && c == other.c && d == other.d);
}

bool plane::operator != ( const plane &other) const
{
	return (a != other.a && b != other.b && c != other.c && d != other.d);
}

} // namespace