#ifndef _PLANE_H_
#define _PLANE_H_

#include <cmath>
#include <iostream>
#include "vector3.h"
using namespace std;

namespace oglclass
{

class plane
{
public:
    float a, b, c, d;

	// constructor
	plane(void) { a = b = c = d = 0.0f; }
	plane(const plane &other);
    plane(const float *other);
    plane(float a_, float b_, float c_, float d_);
    plane(vector3 p0_, vector3 p1_, vector3 p2_);
    plane(vector3 n_, vector3 p_);
    void set(float a_, float b_, float c_, float d_);
    void set(vector3 p0_, vector3 p1_, vector3 p2_);
    void set(vector3 n_, vector3 p_);

	// length & normalize
    float length(void);
    void normalize(void);

	// static utility methods (plane p: a, b, c, d & vector3 v: x, y, z)
    static float dotNormal(const plane &p, const vector3 &v);   // a*x + b*y + c*z + d*0
    static float dotCoord(const plane &p, const vector3 &v);    // a*x + b*y + c*z + d*1
    static vector3& closestPointOnPlane( const plane &p, const vector3 &v);

    // assignment operators...
    plane& operator = (const plane &other);

    // equality operators
	bool operator == (const plane &other) const;
	bool operator != (const plane &other) const;

#ifdef OPENGL
    void draw();
#endif
};

inline std::ostream& operator<< (std::ostream& s, const plane& v)
{ 
	return s << "(" << v.a << ", " << v.b << ", "
                   << v.c << ", " << v.d << ")"; 
}

} // namespace

#endif
