#include "Framework.h"
#include "sound.h"

int sound::numSounds = 0;

sound::sound(void)
{
}

sound::~sound(void)
{
    cleanup();
}

bool sound::init(void)
{
	// initialize Framework
	ALFWInit();

	if (!ALFWInitOpenAL()) {
		ALFWprintf("Failed to initialize OpenAL\n");
		ALFWShutdown();
	}

   	// generate MAX_NUM AL Buffers
	alGenBuffers(MAX_NUM, uiBuffers);
    if (alGetError() != AL_NO_ERROR)
        return false;

	// generate a source to playback the buffer
    alGenSources(MAX_NUM, uiSources);
    if (alGetError() != AL_NO_ERROR)
        return false;

    return true;
}

void sound::cleanup(void)
{
    // stop all sound sources
    for (int i = 0; i < numSounds; i++)
		alSourceStop(uiSources[i]);

    // delete all sound sources & buffers
    //ALFWprintf("After stop\n");
	alDeleteSources(numSounds, &uiSources[0]);
	//ALFWprintf("After delete source\n");
	alDeleteBuffers(numSounds, &uiBuffers[0]);
	//ALFWprintf("After delete buffer\n");

    // shutdown
	ALFWShutdownOpenAL();
	//ALFWprintf("After shutting down OpenAL\n");
	ALFWShutdown();
	//ALFWprintf("After shutting down the framework\n");
}

void sound::setListner(ALfloat * pos, ALfloat * vel, ALfloat * ori)
{
    alListenerfv(AL_POSITION,    pos);
    alListenerfv(AL_VELOCITY,    vel);
    alListenerfv(AL_ORIENTATION, ori);
}

void sound::setSoundSources(char * filename, ALfloat * pos, ALfloat * vel, ALboolean loop)
{
	// load a Wave file into OpenAL Buffer
	if (!ALFWLoadWaveToBuffer(filename, uiBuffers[numSounds])) {
		ALFWprintf("Failed to load sound file\n");
	}

    // attach sound source to buffer
	alSourcei(uiSources[numSounds], AL_BUFFER, uiBuffers[numSounds] );
    alSourcef(uiSources[numSounds], AL_PITCH, 1.0f);
    alSourcef(uiSources[numSounds], AL_GAIN, 1.0f);
	alSourcefv(uiSources[numSounds], AL_POSITION, pos);
	alSourcefv(uiSources[numSounds], AL_VELOCITY, vel);
	alSourcei(uiSources[numSounds], AL_LOOPING, loop);	// AL_TRUE for a looping sound

    // increase sound source index
    numSounds++;
}

void sound::play(int index)
{
    if ((index >= 0) && (index <= numSounds)) {
        printf("play sound #%d\n", index);
	    alSourcePlay(uiSources[index]);
    }
}

void sound::stop(int index)
{
    if ((index >= 0) && (index <= numSounds))
	    alSourceStop(uiSources[index]);
}