#include <windows.h>
#include <stdio.h>
#include <GL/glu.h>
#include "simage.h"
#include "texture1D.h"

namespace oglclass
{

texture1D::texture1D(char *filename, GLint wrap, GLint min, GLint mag) :
                texture(wrap,min,mag)
{
    textureID_ = 0;
    imageData_ = 0;
    rescaledImageData_ = 0;
    defined_ = false;
    createdImage_ = false;
    if (filename)
    {
		imageData_ = simage_read_image(filename, &width_, &height_, &numComponents_);
		createdImage_ = true;
    }
}

texture1D::~texture1D(void)
{
    if (rescaledImageData_)
        simage_free_image(rescaledImageData_);
    if (createdImage_)
        simage_free_image(imageData_);
}

void texture1D::define(void)
{
	GLenum type;
    GLsizei xdim2;
    unsigned char *imgPtr;
    defined_ = true;
    if (!imageData_)
        return;
    xdim2 = 1;
    while (xdim2 <= width_)
        xdim2 *= 2;
    xdim2 /= 2;
    if (width_ != xdim2)
        {
        rescaledImageData_ = simage_resize(imageData_, width_, height_, numComponents_,
                                           xdim2, 1);
        imgPtr = rescaledImageData_;
        }
    else
        imgPtr = imageData_;

	if (numComponents_ == 4) type = GL_RGBA;
	else if (numComponents_ == 3) type = GL_RGB;
	else if (numComponents_ == 2) type = GL_LUMINANCE_ALPHA;
	else if (numComponents_ == 1) type = GL_LUMINANCE;

	if (textureID_ == 0)
        glGenTextures(1, &textureID_);

    glBindTexture(GL_TEXTURE_1D, textureID_);
    if ((minFilter() == GL_NEAREST_MIPMAP_NEAREST) ||
          (minFilter() == GL_NEAREST_MIPMAP_LINEAR) ||
          (minFilter() == GL_LINEAR_MIPMAP_NEAREST) ||
          (minFilter() == GL_LINEAR_MIPMAP_LINEAR))
        gluBuild1DMipmaps(GL_TEXTURE_1D, (GLint) type, xdim2, type,
                          GL_UNSIGNED_BYTE, imgPtr);
    else
        glTexImage1D(GL_TEXTURE_1D, 0, (GLint) type, xdim2, 0, type,
                     GL_UNSIGNED_BYTE, imgPtr);
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,magFilter());
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,minFilter());
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,wrapS());
    glBindTexture(GL_TEXTURE_1D,0);
}

void texture1D::enable(void)
{
    if (!defined_)
        define();
    if (textureID_)
    {
        glBindTexture(GL_TEXTURE_1D, textureID_);
        glEnable(GL_TEXTURE_1D);
    }
    else
    {
        glBindTexture(GL_TEXTURE_1D,0);
        glDisable(GL_TEXTURE_1D);
    }
}

void texture1D::disable(void)
{
    glBindTexture(GL_TEXTURE_1D,0);
    glDisable(GL_TEXTURE_1D);
}

} // namespace
