#include <windows.h>
#include <stdio.h>
#include <GL/glu.h>
#include "simage.h"
#include "texture2D.h"

namespace oglclass
{

texture2D::texture2D(char *filename, int wrap, int min, int mag) :
                texture(wrap,min,mag)
{
	textureID_ = 0;
	imageData_ = 0;
	rescaledImageData_ = 0;
	defined_ = false;
	createdImage_ = false;
	if (filename)
	{
		imageData_ = simage_read_image(filename, &width_, &height_, &numComponents_);
		createdImage_ = true;
	}
}

texture2D::~texture2D(void)
{
    if (rescaledImageData_)
        simage_free_image(rescaledImageData_);
    if (createdImage_)
        simage_free_image(imageData_);
}

void texture2D::define(void)
{
	GLsizei xdim2,ydim2;
	GLenum type;
    unsigned char *imgPtr;
    defined_ = true;
    if (!imageData_)
        return;
    xdim2 = 1;
    while (xdim2 <= width_)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= height_)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((width_ != xdim2) || (height_ != ydim2))
        {
        rescaledImageData_ = simage_resize(imageData_, width_, height_, numComponents_,
                                           xdim2, ydim2);
        imgPtr = rescaledImageData_;
        }
    else
        imgPtr = imageData_;

	if (numComponents_ == 4) type = GL_RGBA;
	else if (numComponents_ == 3) type = GL_RGB;
	else if (numComponents_ == 2) type = GL_LUMINANCE_ALPHA;
	else if (numComponents_ == 1) type = GL_LUMINANCE;

	if (textureID_ == 0)
		glGenTextures(1, &textureID_);

	glBindTexture(GL_TEXTURE_2D, textureID_);
    if ((minFilter() == GL_NEAREST_MIPMAP_NEAREST) ||
          (minFilter() == GL_NEAREST_MIPMAP_LINEAR) ||
          (minFilter() == GL_LINEAR_MIPMAP_NEAREST) ||
          (minFilter() == GL_LINEAR_MIPMAP_LINEAR))
        gluBuild2DMipmaps(GL_TEXTURE_2D, (GLint) type, xdim2, ydim2, type,
                          GL_UNSIGNED_BYTE, imgPtr);
    else
        glTexImage2D(GL_TEXTURE_2D, 0, (GLint) type, xdim2, ydim2, 0, type,
                     GL_UNSIGNED_BYTE, imgPtr);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, magFilter());
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, minFilter());
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, wrapS());
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, wrapT());
    glBindTexture(GL_TEXTURE_2D,0);
}

void texture2D::subload(GLint xoffset,GLint yoffset)
{
    if (!defined_)
        return;

    if (!imageData_)
        return;

    glBindTexture(GL_TEXTURE_2D, textureID_);
    if ((minFilter() == GL_NEAREST_MIPMAP_NEAREST) ||
          (minFilter() == GL_NEAREST_MIPMAP_LINEAR) ||
          (minFilter() == GL_LINEAR_MIPMAP_NEAREST) ||
          (minFilter() == GL_LINEAR_MIPMAP_LINEAR))
        printf("ERROR: texture2D:subload(): subloading a mipmapped texture"
               " is not supported\n");
    else
        glTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset,
                        width_, height_,
                        GL_RGBA, GL_UNSIGNED_BYTE, imageData_);
    glBindTexture(GL_TEXTURE_2D,0);
}

void texture2D::enable(void)
{
	if (!defined_)
		define();
	if (textureID_)
	{
		glBindTexture(GL_TEXTURE_2D, textureID_);
		glEnable(GL_TEXTURE_2D);
	}
	else
	{
		glBindTexture(GL_TEXTURE_2D, 0);
		glDisable(GL_TEXTURE_2D);
	}
}

void texture2D::disable(void)
{
	glBindTexture(GL_TEXTURE_2D, 0);
    glDisable(GL_TEXTURE_2D);
}
/*
void texture2D::setImage(Image& im)
{
	image_ = &im;
}
*/

} // namespace
