#include "transform.h"

namespace oglclass
{

transform::transform(void)
{
	setTranslation(0,0,0);
	setRotation(0,1,0,0);
	setScaling(1);
}

void transform::apply(void)
{
	glTranslatef(translate_[0], translate_[1], translate_[2]);
	glRotatef(rotateAngle_, rotateAxis_[0], rotateAxis_[1], rotateAxis_[2]);
	glScalef(scale_[0], scale_[1], scale_[2]);
}

void transform::pushMatrix(void)
{
	glPushMatrix();
	apply();
}

void transform::popMatrix(void)
{
	glPopMatrix();
}

void transform::setTranslation(const vector3& trans) 
{
	translate_[0] = trans.vec[0];
	translate_[1] = trans.vec[1];
	translate_[2] = trans.vec[2];
}

void transform::setTranslation(GLfloat x,GLfloat y,GLfloat z)
{
	translate_[0] = x;
	translate_[1] = y;
	translate_[2] = z;
}

void transform::setRotation(GLfloat angle,const vector3& axis) 
{
	rotateAngle_ = angle;
	rotateAxis_[0] = axis.vec[0];
	rotateAxis_[1] = axis.vec[1];
	rotateAxis_[2] = axis.vec[2];
}

void transform::setRotation(GLfloat angle,GLfloat x,GLfloat y,GLfloat z)
{
	rotateAngle_ = angle;
	rotateAxis_[0] = x;
	rotateAxis_[1] = y;
	rotateAxis_[2] = z;
}

void transform::setScaling(GLfloat s)
{
	scale_[0] = s;
	scale_[1] = s;
	scale_[2] = s;
}

void transform::setScaling(const vector3& s)
{
	scale_[0] = s.vec[0];
	scale_[1] = s.vec[1];
	scale_[2] = s.vec[2];
}

void transform::setScaling(GLfloat x,GLfloat y,GLfloat z)
{
	scale_[0] = x;
	scale_[1] = y;
	scale_[2] = z;
}

}
