#include <cmath>
#include "vector3.h"

namespace oglclass
{

vector3::vector3( const vector3 &other )
{
    vec[0] = other.vec[0];
    vec[1] = other.vec[1];
    vec[2] = other.vec[2];
}

vector3::vector3( const float *other )
{
    vec[0] = other[0];
    vec[1] = other[1];
    vec[2] = other[2];
}

vector3::vector3( float x_, float y_, float z_ )
{
    vec[0] = x_;
    vec[1] = y_;
    vec[2] = z_;
}

void vector3::set( float x_, float y_, float z_ )
{
    vec[0] = x_;
    vec[1] = y_;
    vec[2] = z_;
}

float vector3::length( void )
{
    return( (float)sqrt( vec[0] * vec[0] + vec[1] * vec[1] + vec[2] * vec[2] ) );
}

void vector3::normalize( void )
{
    float fLength = length();

    vec[0] = vec[0] / fLength;
    vec[1] = vec[1] / fLength;
    vec[2] = vec[2] / fLength;
}

float vector3::distance( const vector3 &v1,  const vector3 &v2  )
{
    float dx = v1.vec[0] - v2.vec[0];
    float dy = v1.vec[1] - v2.vec[1];
    float dz = v1.vec[2] - v2.vec[2];

    return (float)sqrt( dx * dx + dy * dy + dz * dz );
}

float vector3::dotProduct( const vector3 &v1,  const vector3 &v2 )
{
    return (v1.vec[0] * v2.vec[0] + v1.vec[1] * v2.vec[1] + v1.vec[2] * v2.vec[2]);
}

vector3 vector3::crossProduct( const vector3 &v1,  const vector3 &v2 )
{
	vector3 vCrossProduct;

    vCrossProduct.vec[0] = v1.vec[1] * v2.vec[2] - v1.vec[2] * v2.vec[1];
    vCrossProduct.vec[1] = v1.vec[2] * v2.vec[0] - v1.vec[0] * v2.vec[2];
    vCrossProduct.vec[2] = v1.vec[0] * v2.vec[1] - v1.vec[1] * v2.vec[0];

    return vCrossProduct;
}

// operators...
vector3& vector3::operator = ( const vector3 &other )
{
    vec[0] = other.vec[0];
    vec[1] = other.vec[1];
    vec[2] = other.vec[2];

    return *this;
}

vector3& vector3::operator += ( const vector3 &other )
{
    vec[0] += other.vec[0];
    vec[1] += other.vec[1];
    vec[2] += other.vec[2];

    return *this;
}

vector3& vector3::operator -= ( const vector3 &other )
{
    vec[0] -= other.vec[0];
    vec[1] -= other.vec[1];
    vec[2] -= other.vec[2];

    return *this;
}

 vector3& vector3::operator *= ( const float scalar )
{
    vec[0] *= scalar;
    vec[1] *= scalar;
    vec[2] *= scalar;

    return *this;
}

vector3& vector3::operator /= ( const float scalar )
{
    vec[0] /= scalar;
    vec[1] /= scalar;
    vec[2] /= scalar;

    return *this;
}

bool vector3::operator == ( const vector3 &other) const
{
	return (vec[0] == other.vec[0] && vec[1] == other.vec[1] && vec[2] == other.vec[2]);
}

bool vector3::operator != ( const vector3 &other) const
{
	return (vec[0] != other.vec[0] && vec[1] != other.vec[1] && vec[2] != other.vec[2]);
}

vector3 vector3::operator + ( const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] + other.vec[0];
    vResult.vec[1] = vec[1] + other.vec[1];
    vResult.vec[2] = vec[2] + other.vec[2];

    return vResult;
}

vector3 vector3::operator - ( const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] - other.vec[0];
    vResult.vec[1] = vec[1] - other.vec[1];
    vResult.vec[2] = vec[2] - other.vec[2];

    return vResult;
}

vector3 vector3::operator * ( const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] * other.vec[0];
    vResult.vec[1] = vec[1] * other.vec[1];
    vResult.vec[2] = vec[2] * other.vec[2];

    return vResult;
}

vector3 vector3::operator / ( const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] / other.vec[0];
    vResult.vec[1] = vec[1] / other.vec[1];
    vResult.vec[2] = vec[2] / other.vec[2];

    return vResult;
}

vector3 vector3::operator * ( const float scalar )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] * scalar;
    vResult.vec[1] = vec[1] * scalar;
    vResult.vec[2] = vec[2] * scalar;

    return vResult;
}

vector3 vector3::operator / ( const float scalar )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] / scalar;
    vResult.vec[1] = vec[1] / scalar;
    vResult.vec[2] = vec[2] / scalar;

    return vResult;
}

vector3 operator * ( const float scalar, const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = other.vec[0] * scalar;
    vResult.vec[1] = other.vec[1] * scalar;
    vResult.vec[2] = other.vec[2] * scalar;

    return vResult;
}

vector3 operator / ( const float scalar, const vector3 &other )
{
    vector3 vResult(0.0f, 0.0f, 0.0f);

    vResult.vec[0] = other.vec[0] / scalar;
    vResult.vec[1] = other.vec[1] / scalar;
    vResult.vec[2] = other.vec[2] / scalar;

    return vResult;
}

vector3 vector3::operator + ( void ) const
{
    return *this;
}

vector3 vector3::operator - ( void ) const
{
    vector3 vResult(-vec[0], -vec[1], -vec[2]);

    return vResult;
}

float& vector3::operator[] (int i)
{
	if (i >= 0 && i <= 3)
		return vec[i];
}

} // namespace