#include "vector4.h"

namespace oglclass
{

vector4::vector4( const vector4 &other )
{
    vec[0] = other.vec[0];
    vec[1] = other.vec[1];
    vec[2] = other.vec[2];
    vec[3] = other.vec[3];
}

vector4::vector4( const float *other )
{
    vec[0] = other[0];
    vec[1] = other[1];
    vec[2] = other[2];
    vec[3] = other[3];
}

vector4::vector4( float x_, float y_, float z_, float w_)
{
    vec[0] = x_;
    vec[1] = y_;
    vec[2] = z_;
    vec[3] = w_;
}

void vector4::set( float x_, float y_, float z_, float w_ )
{
    vec[0] = x_;
    vec[1] = y_;
    vec[2] = z_;
    vec[3] = w_;
}

float vector4::length( void )
{
    return( (float)sqrt( vec[0] * vec[0] + vec[1] * vec[1] + vec[2] * vec[2] + vec[3] * vec[3]) );
}

void vector4::normalize( void )
{
    float fLength = length();

    vec[0] = vec[0] / fLength;
    vec[1] = vec[1] / fLength;
    vec[2] = vec[2] / fLength;
    vec[3] = vec[3] / fLength;
}

float vector4::distance( const vector4 &v1,  const vector4 &v2  )
{
    float dx = v1.vec[0] - v2.vec[0];
    float dy = v1.vec[1] - v2.vec[1];
    float dz = v1.vec[2] - v2.vec[2];
    float dw = v1.vec[3] - v2.vec[3];

    return (float)sqrt( dx * dx + dy * dy + dz * dz + dw * dw );
}

float vector4::dotProduct( const vector4 &v1,  const vector4 &v2 )
{
    return ( v1.vec[0] * v2.vec[0] + v1.vec[1] * v2.vec[1] + v1.vec[2] * v2.vec[2] + v1.vec[3] * v2.vec[3] );
}

// operators...
vector4& vector4::operator = ( const vector4 &other )
{
    vec[0] = other.vec[0];
    vec[1] = other.vec[1];
    vec[2] = other.vec[2];
    vec[3] = other.vec[3];

    return *this;
}

vector4& vector4::operator += ( const vector4 &other )
{
    vec[0] += other.vec[0];
    vec[1] += other.vec[1];
    vec[2] += other.vec[2];
    vec[3] += other.vec[3];

    return *this;
}

vector4& vector4::operator -= ( const vector4 &other )
{
    vec[0] -= other.vec[0];
    vec[1] -= other.vec[1];
    vec[2] -= other.vec[2];
    vec[3] -= other.vec[3];

    return *this;
}

vector4& vector4::operator *= ( const float scalar )
{
    vec[0] *= scalar;
    vec[1] *= scalar;
    vec[2] *= scalar;
    vec[3] *= scalar;

    return *this;
}

vector4& vector4::operator /= ( const float scalar )
{
    vec[0] /= scalar;
    vec[1] /= scalar;
    vec[2] /= scalar;
    vec[3] /= scalar;

    return *this;
}

bool vector4::operator == ( const vector4 &other) const
{
	return (vec[0] == other.vec[0] && vec[1] == other.vec[1] && vec[2] == other.vec[2] && vec[3] == other.vec[3]);
}

bool vector4::operator != ( const vector4 &other) const
{
	return (vec[0] != other.vec[0] && vec[1] != other.vec[1] && vec[2] != other.vec[2] && vec[3] != other.vec[3]);
}

vector4 vector4::operator + ( const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] + other.vec[0];
    vResult.vec[1] = vec[1] + other.vec[1];
    vResult.vec[2] = vec[2] + other.vec[2];
    vResult.vec[3] = vec[3] + other.vec[3];

    return vResult;
}

vector4 vector4::operator - ( const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] - other.vec[0];
    vResult.vec[1] = vec[1] - other.vec[1];
    vResult.vec[2] = vec[2] - other.vec[2];
    vResult.vec[3] = vec[3] - other.vec[3];

    return vResult;
}

vector4 vector4::operator * ( const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] * other.vec[0];
    vResult.vec[1] = vec[1] * other.vec[1];
    vResult.vec[2] = vec[2] * other.vec[2];
    vResult.vec[3] = vec[3] * other.vec[3];

    return vResult;
}

vector4 vector4::operator / ( const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] / other.vec[0];
    vResult.vec[1] = vec[1] / other.vec[1];
    vResult.vec[2] = vec[2] / other.vec[2];
    vResult.vec[3] = vec[3] / other.vec[3];

    return vResult;
}

vector4 vector4::operator * ( const float scalar )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] * scalar;
    vResult.vec[1] = vec[1] * scalar;
    vResult.vec[2] = vec[2] * scalar;
    vResult.vec[3] = vec[3] * scalar;

    return vResult;
}

vector4 vector4::operator / ( const float scalar )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = vec[0] / scalar;
    vResult.vec[1] = vec[1] / scalar;
    vResult.vec[2] = vec[2] / scalar;
    vResult.vec[3] = vec[3] / scalar;

    return vResult;
}

vector4 operator * ( const float scalar, const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = other.vec[0] * scalar;
    vResult.vec[1] = other.vec[1] * scalar;
    vResult.vec[2] = other.vec[2] * scalar;
    vResult.vec[3] = other.vec[3] * scalar;

    return vResult;
}

vector4 operator / ( const float scalar, const vector4 &other )
{
    vector4 vResult(0.0f, 0.0f, 0.0f, 0.0f);

    vResult.vec[0] = other.vec[0] / scalar;
    vResult.vec[1] = other.vec[1] / scalar;
    vResult.vec[2] = other.vec[2] / scalar;
    vResult.vec[3] = other.vec[3] / scalar;

    return vResult;
}

vector4 vector4::operator + ( void ) const
{
    return *this;
}

vector4 vector4::operator - ( void ) const
{
    vector4 vResult(-vec[0], -vec[1], -vec[2], -vec[3]);

    return vResult;
}

float& vector4::operator[] (int i)
{
	if (i >= 0 && i <=3)
		return vec[i];
}

} // namespace