#ifndef _VECTOR4_H_
#define _VECTOR4_H_

#include <cmath>
#include <iostream>
using namespace std;

namespace oglclass
{

class vector4
{
public:
    float vec[4];

	// constructor
	vector4(void) { vec[0] = vec[1] = vec[2] = vec[3] = 0.0f; }
	vector4(const vector4 &other);
    vector4(const float *other);
    vector4(float x_, float y_, float z_, float w_);
    void set(float x_, float y_, float z_, float w_);

	// length & normalize
    float length(void);
    void normalize(void);

	// static utility methods
    static float distance(const vector4 &v1, const vector4 &v2);
    static float dotProduct(const vector4 &v1, const vector4 &v2 );

    // assignment operators...
    vector4& operator = (const vector4 &other);
    vector4& operator += (const vector4 &other);
    vector4& operator -= (const vector4 &other);
    vector4& operator *= (const float scalar);
    vector4& operator /= (const float scalar);

	// equality operators
	bool operator == (const vector4 &other) const;
	bool operator != (const vector4 &other) const;

	// vector operators
    vector4 operator + (const vector4 &other);
    vector4 operator - (const vector4 &other);
    vector4 operator * (const vector4 &other);
    vector4 operator / (const vector4 &other);

    vector4 operator * (const float scalar);
    friend vector4 operator * (const float scalar, const vector4 &other);
    vector4 operator / (const float scalar);
    friend vector4 operator / (const float scalar, const vector4 &other);
   
    vector4 operator + (void) const;
    vector4 operator - (void) const;

	float& operator[] (int i);
};

inline std::ostream& operator<< (std::ostream& s, const vector4& v)
{ 
	return s << "(" << v.vec[0] << ", " << v.vec[1] << ", "
                   << v.vec[2] << ", " << v.vec[3] << ")"; 
}

} // namespace

#endif
